@@ -74,10 +74,11 @@ public class AsyncContextImpl implements AsyncContext, AsyncContextCallback {
     private long timeout = -1;
     private AsyncEvent event = null;
     private volatile Request request;
-    AtomicBoolean hasProcessedError = new AtomicBoolean(false);
+    private final AtomicBoolean hasErrorProcessingStarted = new AtomicBoolean(false);
+    private final AtomicBoolean hasOnErrorReturned = new AtomicBoolean(false);
 
     public AsyncContextImpl(Request request) {
-        if (log.isDebugEnabled()) {
+        if (log.isTraceEnabled()) {
             logDebug("Constructor");
         }
         this.request = request;
@@ -85,7 +86,7 @@ public class AsyncContextImpl implements AsyncContext, AsyncContextCallback {
 
     @Override
     public void complete() {
-        if (log.isDebugEnabled()) {
+        if (log.isTraceEnabled()) {
             logDebug("complete   ");
         }
         check();
@@ -94,8 +95,8 @@ public class AsyncContextImpl implements AsyncContext, AsyncContextCallback {
 
     @Override
     public void fireOnComplete() {
-        if (log.isDebugEnabled()) {
-            log.debug(sm.getString("asyncContextImpl.fireOnComplete"));
+        if (log.isTraceEnabled()) {
+            log.trace(sm.getString("asyncContextImpl.fireOnComplete"));
         }
         List<AsyncListenerWrapper> listenersCopy = new ArrayList<>(listeners);
 
@@ -124,8 +125,8 @@ public class AsyncContextImpl implements AsyncContext, AsyncContextCallback {
         Context context = this.context;
 
         if (result.get()) {
-            if (log.isDebugEnabled()) {
-                log.debug(sm.getString("asyncContextImpl.fireOnTimeout"));
+            if (log.isTraceEnabled()) {
+                log.trace(sm.getString("asyncContextImpl.fireOnTimeout"));
             }
             ClassLoader oldCL = context.bind(false, null);
             try {
@@ -178,7 +179,7 @@ public class AsyncContextImpl implements AsyncContext, AsyncContextCallback {
     @Override
     public void dispatch(ServletContext servletContext, String path) {
         synchronized (asyncContextLock) {
-            if (log.isDebugEnabled()) {
+            if (log.isTraceEnabled()) {
                 logDebug("dispatch   ");
             }
             check();
@@ -225,7 +226,7 @@ public class AsyncContextImpl implements AsyncContext, AsyncContextCallback {
 
     @Override
     public void start(final Runnable run) {
-        if (log.isDebugEnabled()) {
+        if (log.isTraceEnabled()) {
             logDebug("start      ");
         }
         check();
@@ -270,7 +271,7 @@ public class AsyncContextImpl implements AsyncContext, AsyncContextCallback {
     }
 
     public void recycle() {
-        if (log.isDebugEnabled()) {
+        if (log.isTraceEnabled()) {
             logDebug("recycle    ");
         }
         context = null;
@@ -281,7 +282,8 @@ public class AsyncContextImpl implements AsyncContext, AsyncContextCallback {
         request = null;
         clearServletRequestResponse();
         timeout = -1;
-        hasProcessedError.set(false);
+        hasErrorProcessingStarted.set(false);
+        hasOnErrorReturned.set(false);
     }
 
     private void clearServletRequestResponse() {
@@ -291,6 +293,8 @@ public class AsyncContextImpl implements AsyncContext, AsyncContextCallback {
 
     public boolean isStarted() {
         AtomicBoolean result = new AtomicBoolean(false);
+        Request request = this.request;
+        check();
         request.getCoyoteRequest().action(ActionCode.ASYNC_IS_STARTED, result);
         return result.get();
     }
@@ -310,8 +314,8 @@ public class AsyncContextImpl implements AsyncContext, AsyncContextCallback {
 
             List<AsyncListenerWrapper> listenersCopy = new ArrayList<>(listeners);
             listeners.clear();
-            if (log.isDebugEnabled()) {
-                log.debug(sm.getString("asyncContextImpl.fireOnStartAsync"));
+            if (log.isTraceEnabled()) {
+                log.trace(sm.getString("asyncContextImpl.fireOnStartAsync"));
             }
             for (AsyncListenerWrapper listener : listenersCopy) {
                 try {
@@ -331,7 +335,7 @@ public class AsyncContextImpl implements AsyncContext, AsyncContextCallback {
     }
 
     protected void doInternalDispatch() throws ServletException, IOException {
-        if (log.isDebugEnabled()) {
+        if (log.isTraceEnabled()) {
             logDebug("intDispatch");
         }
         try {
@@ -380,7 +384,7 @@ public class AsyncContextImpl implements AsyncContext, AsyncContextCallback {
 
 
     public void setErrorState(Throwable t, boolean fireOnError) {
-        if (!hasProcessedError.compareAndSet(false, true)) {
+        if (!hasErrorProcessingStarted.compareAndSet(false, true)) {
             // Skip duplicate error processing
             return;
         }
@@ -390,8 +394,8 @@ public class AsyncContextImpl implements AsyncContext, AsyncContextCallback {
         request.getCoyoteRequest().action(ActionCode.ASYNC_ERROR, null);
 
         if (fireOnError) {
-            if (log.isDebugEnabled()) {
-                log.debug(sm.getString("asyncContextImpl.fireOnError"));
+            if (log.isTraceEnabled()) {
+                log.trace(sm.getString("asyncContextImpl.fireOnError"));
             }
             AsyncEvent errorEvent =
                     new AsyncEvent(event.getAsyncContext(), event.getSuppliedRequest(), event.getSuppliedResponse(), t);
@@ -402,6 +406,8 @@ public class AsyncContextImpl implements AsyncContext, AsyncContextCallback {
                 } catch (Throwable t2) {
                     ExceptionUtils.handleThrowable(t2);
                     log.warn(sm.getString("asyncContextImpl.onErrorError", listener.getClass().getName()), t2);
+                } finally {
+                    hasOnErrorReturned.set(true);
                 }
             }
         }
@@ -501,10 +507,19 @@ public class AsyncContextImpl implements AsyncContext, AsyncContextCallback {
     }
 
     private void check() {
+        Request request = this.request;
         if (request == null) {
             // AsyncContext has been recycled and should not be being used
             throw new IllegalStateException(sm.getString("asyncContextImpl.requestEnded"));
         }
+        if (hasOnErrorReturned.get() && !request.getCoyoteRequest().isRequestThread()) {
+            /*
+             * Non-container thread is trying to use the AsyncContext after an error has occurred and the call to
+             * AsyncListener.onError() has returned. At this point, the non-container thread should not be trying to use
+             * the AsyncContext due to possible race conditions.
+             */
+            throw new IllegalStateException(sm.getString("asyncContextImpl.afterOnError"));
+        }
     }
 
     private static class DebugException extends Exception {
