@@ -512,7 +512,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
      * this container. Unexpected throwables will be caught and logged.
      */
     @Override
-    public void backgroundProcess() {
+    public synchronized void backgroundProcess() {
         super.backgroundProcess();
 
         if (!getState().isAvailable()) {
@@ -643,8 +643,8 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
             synchronized (this) {
                 if (instance == null) {
                     try {
-                        if (log.isDebugEnabled()) {
-                            log.debug("Allocating instance");
+                        if (log.isTraceEnabled()) {
+                            log.trace("Allocating instance");
                         }
                         instance = loadServlet();
                         newInstance = true;
@@ -1142,21 +1142,12 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     // -------------------------------------------------- ServletConfig Methods
 
 
-    /**
-     * @return the initialization parameter value for the specified name, if any; otherwise return <code>null</code>.
-     *
-     * @param name Name of the initialization parameter to retrieve
-     */
     @Override
     public String getInitParameter(String name) {
         return findInitParameter(name);
     }
 
 
-    /**
-     * @return the set of initialization parameter names defined for this servlet. If none are defined, an empty
-     *             Enumeration is returned.
-     */
     @Override
     public Enumeration<String> getInitParameterNames() {
 
@@ -1170,9 +1161,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * @return the servlet context with which this servlet is associated.
-     */
     @Override
     public ServletContext getServletContext() {
         if (parent == null) {
@@ -1185,9 +1173,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * @return the name of this servlet.
-     */
     @Override
     public String getServletName() {
         return getName();
@@ -1319,7 +1304,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
      *                                   used
      */
     @Override
-    protected synchronized void startInternal() throws LifecycleException {
+    protected void startInternal() throws LifecycleException {
 
         // Send j2ee.state.starting notification
         if (this.getObjectName() != null) {
@@ -1349,7 +1334,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
      *                                   used
      */
     @Override
-    protected synchronized void stopInternal() throws LifecycleException {
+    protected void stopInternal() throws LifecycleException {
 
         setAvailable(Long.MAX_VALUE);
 
@@ -1468,8 +1453,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
      */
     @Override
     public MBeanNotificationInfo[] getNotificationInfo() {
-        // FIXME: we not send j2ee.state.failed
-        // FIXME: we not send j2ee.attribute.changed
         if (notificationInfo == null) {
             notificationInfo = new MBeanNotificationInfo[] {
                     new MBeanNotificationInfo(new String[] { "j2ee.object.created" }, Notification.class.getName(),
