@@ -16,8 +16,9 @@
  */
 package org.apache.catalina.loader;
 
+import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
 import java.io.File;
-import java.io.FileNotFoundException;
 import java.io.FilePermission;
 import java.io.IOException;
 import java.io.InputStream;
@@ -25,6 +26,7 @@ import java.lang.instrument.ClassFileTransformer;
 import java.lang.instrument.IllegalClassFormatException;
 import java.lang.ref.Reference;
 import java.lang.reflect.Field;
+import java.lang.reflect.InaccessibleObjectException;
 import java.lang.reflect.Method;
 import java.net.URI;
 import java.net.URISyntaxException;
@@ -74,8 +76,6 @@ import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.InstrumentableClassLoader;
 import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.IntrospectionUtils;
-import org.apache.tomcat.util.buf.ToStringUtil;
-import org.apache.tomcat.util.collections.ConcurrentLruCache;
 import org.apache.tomcat.util.compat.JreCompat;
 import org.apache.tomcat.util.res.StringManager;
 import org.apache.tomcat.util.security.PermissionCheck;
@@ -316,7 +316,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
 
     /**
      * Enables the RMI Target memory leak detection to be controlled. This is necessary since the detection can only
-     * work on Java 9 if some of the modularity checks are disabled.
+     * work if some of the modularity checks are disabled.
      */
     private boolean clearReferencesRmiTargets = true;
 
@@ -385,44 +385,14 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
     /**
      * Repositories managed by this class rather than the super class.
      */
-    private final List<URL> localRepositories = new ArrayList<>();
+    private List<URL> localRepositories = new ArrayList<>();
 
 
     private volatile LifecycleState state = LifecycleState.NEW;
 
-    /*
-     * Class resources are not cached since they are loaded on first use and the resource is then no longer required. It
-     * does help, however, to cache classes that are not found as in some scenarios the same class will be searched for
-     * many times and the greater the number of JARs/classes, the longer that lookup will take.
-     */
-    private final ConcurrentLruCache<String> notFoundClassResources = new ConcurrentLruCache<>(1000);
-
 
     // ------------------------------------------------------------- Properties
 
-    public void setNotFoundClassResourceCacheSize(int notFoundClassResourceCacheSize) {
-        notFoundClassResources.setLimit(notFoundClassResourceCacheSize);
-    }
-
-
-    public int getNotFoundClassResourceCacheSize() {
-        return notFoundClassResources.getLimit();
-    }
-
-
-    /**
-     * Unused. Always returns {@code null}.
-     *
-     * @return associated resources.
-     *
-     * @deprecated This will be removed in Tomcat 10.1.x onwards
-     */
-    @Deprecated
-    public WebResourceRoot getResources() {
-        return null;
-    }
-
-
     /**
      * Set associated resources.
      *
@@ -636,6 +606,12 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
 
     // ------------------------------------------------------- Reloader Methods
 
+    /**
+     * Adds the specified class file transformer to this class loader. The transformer will then be able to modify the
+     * bytecode of any classes loaded by this class loader after the invocation of this method.
+     *
+     * @param transformer The transformer to add to the class loader
+     */
     @Override
     public void addTransformer(ClassFileTransformer transformer) {
 
@@ -654,6 +630,13 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
         log.info(sm.getString("webappClassLoader.addTransformer", transformer, getContextName()));
     }
 
+    /**
+     * Removes the specified class file transformer from this class loader. It will no longer be able to modify the byte
+     * code of any classes loaded by the class loader after the invocation of this method. However, any classes already
+     * modified by this transformer will remain transformed.
+     *
+     * @param transformer The transformer to remove
+     */
     @Override
     public void removeTransformer(ClassFileTransformer transformer) {
 
@@ -749,7 +732,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
             sb.append(this.parent.toString());
             sb.append("\r\n");
         }
-        if (!this.transformers.isEmpty()) {
+        if (this.transformers.size() > 0) {
             sb.append("----------> Class file transformers:\r\n");
             for (ClassFileTransformer transformer : this.transformers) {
                 sb.append(transformer).append("\r\n");
@@ -784,12 +767,6 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
 
         checkStateForClassLoading(name);
 
-        if (name == null) {
-            throw new ClassNotFoundException("null");
-        }
-
-        String path = binaryNameToPath(name, true);
-
         // (1) Permission to define this class when using a SecurityManager
         if (securityManager != null) {
             int i = name.lastIndexOf('.');
@@ -815,14 +792,25 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
             if (log.isTraceEnabled()) {
                 log.trace("      findClassInternal(" + name + ")");
             }
-            if (!notFoundClassResources.contains(path)) {
+            try {
+                if (securityManager != null) {
+                    PrivilegedAction<Class<?>> dp = new PrivilegedFindClassByName(name);
+                    clazz = AccessController.doPrivileged(dp);
+                } else {
+                    clazz = findClassInternal(name);
+                }
+            } catch (AccessControlException ace) {
+                log.warn(sm.getString("webappClassLoader.securityException", name, ace.getMessage()), ace);
+                throw new ClassNotFoundException(name, ace);
+            } catch (RuntimeException e) {
+                if (log.isTraceEnabled()) {
+                    log.trace("      -->RuntimeException Rethrown", e);
+                }
+                throw e;
+            }
+            if (clazz == null && hasExternalRepositories) {
                 try {
-                    if (securityManager != null) {
-                        PrivilegedAction<Class<?>> dp = new PrivilegedFindClassByName(name);
-                        clazz = AccessController.doPrivileged(dp);
-                    } else {
-                        clazz = findClassInternal(name);
-                    }
+                    clazz = super.findClass(name);
                 } catch (AccessControlException ace) {
                     log.warn(sm.getString("webappClassLoader.securityException", name, ace.getMessage()), ace);
                     throw new ClassNotFoundException(name, ace);
@@ -832,34 +820,19 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
                     }
                     throw e;
                 }
-                if (clazz == null && hasExternalRepositories) {
-                    try {
-                        clazz = super.findClass(name);
-                    } catch (AccessControlException ace) {
-                        log.warn(sm.getString("webappClassLoader.securityException", name, ace.getMessage()), ace);
-                        throw new ClassNotFoundException(name, ace);
-                    } catch (RuntimeException e) {
-                        if (log.isTraceEnabled()) {
-                            log.trace("      -->RuntimeException Rethrown", e);
-                        }
-                        throw e;
-                    }
+            }
+            if (clazz == null) {
+                if (log.isTraceEnabled()) {
+                    log.trace("    --> Returning ClassNotFoundException");
                 }
+                throw new ClassNotFoundException(name);
             }
         } catch (ClassNotFoundException e) {
             if (log.isTraceEnabled()) {
                 log.trace("    --> Passing on ClassNotFoundException");
             }
-            notFoundClassResources.add(path);
             throw e;
         }
-        if (clazz == null) {
-            if (log.isTraceEnabled()) {
-                log.trace("    --> Returning ClassNotFoundException");
-            }
-            notFoundClassResources.add(path);
-            throw new ClassNotFoundException(name);
-        }
 
         // Return the class we have located
         if (log.isTraceEnabled()) {
@@ -899,21 +872,14 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
 
         String path = nameToPath(name);
 
-        if (!notFoundClassResources.contains(path)) {
-            WebResource resource = resources.getClassLoaderResource(path);
-            if (resource.exists()) {
-                url = resource.getURL();
-                trackLastModified(path, resource);
-            }
+        WebResource resource = resources.getClassLoaderResource(path);
+        if (resource.exists()) {
+            url = resource.getURL();
+            trackLastModified(path, resource);
+        }
 
-            if (url == null && hasExternalRepositories) {
-                url = super.findResource(name);
-            }
-
-            // Skip caching results for invalid names (it might mask lookups for valid ones)
-            if (url == null && !name.isEmpty() && name.charAt(0) != '/') {
-                notFoundClassResources.add(path);
-            }
+        if (url == null && hasExternalRepositories) {
+            url = super.findResource(name);
         }
 
         if (log.isTraceEnabled()) {
@@ -939,6 +905,14 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
     }
 
 
+    /**
+     * Return an enumeration of <code>URLs</code> representing all of the resources with the given name. If no resources
+     * with this name are found, return an empty enumeration.
+     *
+     * @param name Name of the resources to be found
+     *
+     * @exception IOException if an input/output error occurs
+     */
     @Override
     public Enumeration<URL> findResources(String name) throws IOException {
 
@@ -996,7 +970,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
 
         checkStateForResourceLoading(name);
 
-        URL url;
+        URL url = null;
 
         boolean delegateFirst = delegate || filter(name, false);
 
@@ -1100,32 +1074,60 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
             log.trace("  Searching local repositories");
         }
         String path = nameToPath(name);
-        if (!notFoundClassResources.contains(path)) {
-            WebResource resource = resources.getClassLoaderResource(path);
-            if (resource.exists()) {
-                stream = resource.getInputStream();
-                trackLastModified(path, resource);
-            }
-            try {
-                if (hasExternalRepositories && stream == null) {
-                    URL url = super.findResource(name);
-                    if (url != null) {
-                        stream = url.openStream();
+        WebResource resource = resources.getClassLoaderResource(path);
+        if (resource.exists()) {
+            stream = resource.getInputStream();
+            // Filter out .class resources through the ClassFileTranformer
+            if (name.endsWith(CLASS_FILE_SUFFIX) && transformers.size() > 0) {
+                // If the resource is a class, decorate it with any attached transformers
+                ByteArrayOutputStream baos = new ByteArrayOutputStream();
+                byte[] buf = new byte[8192];
+                int numRead;
+                try {
+                    while ((numRead = stream.read(buf)) >= 0) {
+                        baos.write(buf, 0, numRead);
+                    }
+                } catch (IOException e) {
+                    log.error(sm.getString("webappClassLoader.transformError", name), e);
+                    return null;
+                } finally {
+                    try {
+                        stream.close();
+                    } catch (IOException e) {
                     }
                 }
-            } catch (IOException e) {
-                // Ignore
-            }
-            if (stream != null) {
-                if (log.isTraceEnabled()) {
-                    log.trace("  --> Returning stream from local");
+                byte[] binaryContent = baos.toByteArray();
+                String internalName = path.substring(1, path.length() - CLASS_FILE_SUFFIX.length());
+                for (ClassFileTransformer transformer : this.transformers) {
+                    try {
+                        byte[] transformed = transformer.transform(this, internalName, null, null, binaryContent);
+                        if (transformed != null) {
+                            binaryContent = transformed;
+                        }
+                    } catch (IllegalClassFormatException e) {
+                        log.error(sm.getString("webappClassLoader.transformError", name), e);
+                        return null;
+                    }
                 }
-                return stream;
+                stream = new ByteArrayInputStream(binaryContent);
             }
-            // Skip caching results for invalid names (it might mask lookups for valid ones)
-            if (!name.isEmpty() && name.charAt(0) != '/') {
-                notFoundClassResources.add(path);
+            trackLastModified(path, resource);
+        }
+        try {
+            if (hasExternalRepositories && stream == null) {
+                URL url = super.findResource(name);
+                if (url != null) {
+                    stream = url.openStream();
+                }
             }
+        } catch (IOException e) {
+            // Ignore
+        }
+        if (stream != null) {
+            if (log.isTraceEnabled()) {
+                log.trace("  --> Returning stream from local");
+            }
+            return stream;
         }
 
         // (3) Delegate to parent unconditionally
@@ -1190,7 +1192,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
             if (log.isTraceEnabled()) {
                 log.trace("loadClass(" + name + ", " + resolve + ")");
             }
-            Class<?> clazz;
+            Class<?> clazz = null;
 
             // Log access to stopped class loader
             checkStateForClassLoading(name);
@@ -1348,9 +1350,6 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
             }
         }
 
-        if (log.isDebugEnabled()) {
-            log.debug(ToStringUtil.classPathForCNFE(this));
-        }
         throw new ClassNotFoundException(name);
     }
 
@@ -1440,24 +1439,43 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
     // ------------------------------------------------------ Lifecycle Methods
 
 
+    /**
+     * Add a lifecycle event listener to this component.
+     *
+     * @param listener The listener to add
+     */
     @Override
     public void addLifecycleListener(LifecycleListener listener) {
         // NOOP
     }
 
 
+    /**
+     * Get the lifecycle listeners associated with this lifecycle. If this Lifecycle has no listeners registered, a
+     * zero-length array is returned.
+     */
     @Override
     public LifecycleListener[] findLifecycleListeners() {
         return new LifecycleListener[0];
     }
 
 
+    /**
+     * Remove a lifecycle event listener from this component.
+     *
+     * @param listener The listener to remove
+     */
     @Override
     public void removeLifecycleListener(LifecycleListener listener) {
         // NOOP
     }
 
 
+    /**
+     * Obtain the current state of the source component.
+     *
+     * @return The current state of the source component.
+     */
     @Override
     public LifecycleState getState() {
         return state;
@@ -1476,6 +1494,11 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
     }
 
 
+    /**
+     * Start the class loader.
+     *
+     * @exception LifecycleException if a lifecycle error occurs
+     */
     @Override
     public void start() throws LifecycleException {
 
@@ -1499,6 +1522,11 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
     }
 
 
+    /**
+     * Stop the class loader.
+     *
+     * @exception LifecycleException if a lifecycle error occurs
+     */
     @Override
     public void stop() throws LifecycleException {
 
@@ -1511,7 +1539,6 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
         state = LifecycleState.STOPPING;
 
         resourceEntries.clear();
-        notFoundClassResources.clear();
         jarModificationTimes.clear();
         resources = null;
 
@@ -1625,9 +1652,6 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
         byte[] classBytes = new byte[2048];
         int offset = 0;
         try (InputStream is = getResourceAsStream("org/apache/catalina/loader/JdbcLeakPrevention.class")) {
-            if (is == null) {
-                throw new FileNotFoundException("org/apache/catalina/loader/JdbcLeakPrevention.class");
-            }
             int read = is.read(classBytes, offset, classBytes.length - offset);
             while (read > -1) {
                 offset += read;
@@ -1726,9 +1750,8 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
                             ((java.util.concurrent.ThreadPoolExecutor) executor).shutdownNow();
                             usingExecutor = true;
                         }
-                    } catch (NoSuchFieldException | IllegalAccessException | RuntimeException e) {
-                        // InaccessibleObjectException is only available in Java 9+,
-                        // swapped for RuntimeException
+                    } catch (SecurityException | NoSuchFieldException | IllegalArgumentException |
+                            IllegalAccessException | InaccessibleObjectException e) {
                         log.warn(sm.getString("webappClassLoader.stopThreadFail", thread.getName(), getContextName()),
                                 e);
                     }
@@ -1753,7 +1776,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
         // when the executor was shut down or the thread was interrupted but
         // that depends on the thread correctly handling the interrupt. Check
         // each thread and if any are still running give all threads up to a
-        // total of 2 seconds to shut down.
+        // total of 2 seconds to shutdown.
         int count = 0;
         for (Thread t : threadsToStop) {
             while (t.isAlive() && count < 100) {
@@ -1784,7 +1807,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
 
         StackTraceElement[] elements = thread.getStackTrace();
 
-        if (elements.length == 0) {
+        if (elements == null || elements.length == 0) {
             // Must have stopped already. Too late to ignore it. Assume not a
             // request processing thread.
             return false;
@@ -1890,17 +1913,12 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
                     }
                 }
             }
+        } catch (InaccessibleObjectException e) {
+            // Must be running on without the necessary command line options.
+            log.warn(sm.getString("webappClassLoader.addExportsThreadLocal", getCurrentModuleName()));
         } catch (Throwable t) {
-            JreCompat jreCompat = JreCompat.getInstance();
-            if (jreCompat.isInstanceOfInaccessibleObjectException(t)) {
-                // Must be running on Java 9 without the necessary command line
-                // options.
-                String currentModule = JreCompat.getInstance().getModuleName(this.getClass());
-                log.warn(sm.getString("webappClassLoader.addExportsThreadLocal", currentModule));
-            } else {
-                ExceptionUtils.handleThrowable(t);
-                log.warn(sm.getString("webappClassLoader.checkThreadLocalsForLeaksFail", getContextName()), t);
-            }
+            ExceptionUtils.handleThrowable(t);
+            log.warn(sm.getString("webappClassLoader.checkThreadLocalsForLeaksFail", getContextName()), t);
         }
     }
 
@@ -2028,8 +2046,8 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
         return false;
     }
 
-    /**
-     * @return the current threads as an array.
+    /*
+     * Get the set of current threads as an array.
      */
     private Thread[] getThreads() {
         // Get the current thread group
@@ -2130,17 +2148,9 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
             log.info(sm.getString("webappClassLoader.clearRmiInfo", getContextName()), e);
         } catch (SecurityException | NoSuchFieldException | IllegalArgumentException | IllegalAccessException e) {
             log.warn(sm.getString("webappClassLoader.clearRmiFail", getContextName()), e);
-        } catch (Exception e) {
-            JreCompat jreCompat = JreCompat.getInstance();
-            if (jreCompat.isInstanceOfInaccessibleObjectException(e)) {
-                // Must be running on Java 9 without the necessary command line
-                // options.
-                String currentModule = JreCompat.getInstance().getModuleName(this.getClass());
-                log.warn(sm.getString("webappClassLoader.addExportsRmi", currentModule));
-            } else {
-                // Re-throw all other exceptions
-                throw e;
-            }
+        } catch (InaccessibleObjectException e) {
+            // Must be running on without the necessary command line options.
+            log.warn(sm.getString("webappClassLoader.addExportsRmi", getCurrentModuleName()));
         }
     }
 
@@ -2158,22 +2168,22 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
             clearCache(clazz, "reflectors");
         } catch (ReflectiveOperationException | SecurityException | ClassCastException e) {
             log.warn(sm.getString("webappClassLoader.clearObjectStreamClassCachesFail", getContextName()), e);
-        } catch (Exception e) {
-            JreCompat jreCompat = JreCompat.getInstance();
-            if (jreCompat.isInstanceOfInaccessibleObjectException(e)) {
-                // Must be running on Java 9 without the necessary command line
-                // options.
-                String currentModule = JreCompat.getInstance().getModuleName(this.getClass());
-                log.warn(sm.getString("webappClassLoader.addExportsJavaIo", currentModule));
-                return;
-            } else {
-                // Re-throw all other exceptions
-                throw e;
-            }
+        } catch (InaccessibleObjectException e) {
+            // Must be running on without the necessary command line options.
+            log.warn(sm.getString("webappClassLoader.addExportsJavaIo", getCurrentModuleName()));
         }
     }
 
 
+    private String getCurrentModuleName() {
+        String moduleName = this.getClass().getModule().getName();
+        if (moduleName == null) {
+            moduleName = "ALL-UNNAMED";
+        }
+        return moduleName;
+    }
+
+
     private void clearCache(Class<?> target, String mapName)
             throws ReflectiveOperationException, SecurityException, ClassCastException {
         Field f = target.getDeclaredField(mapName);
@@ -2204,6 +2214,11 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
      *
      * @return the loaded class, or null if the class isn't found
      */
+    /*
+     * The use of getPackage() is appropriate given that the code is checking if the package is sealed. Therefore,
+     * parent class loaders need to be checked.
+     */
+    @SuppressWarnings("deprecation")
     protected Class<?> findClassInternal(String name) {
 
         checkStateForResourceLoading(name);
@@ -2213,11 +2228,6 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
         }
         String path = binaryNameToPath(name, true);
 
-        return findClassInternal(name, path);
-    }
-
-
-    private Class<?> findClassInternal(String name, String path) {
         ResourceEntry entry = resourceEntries.get(path);
         WebResource resource = null;
 
@@ -2274,7 +2284,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
             URL codeBase = resource.getCodeBase();
             Certificate[] certificates = resource.getCertificates();
 
-            if (!transformers.isEmpty()) {
+            if (transformers.size() > 0) {
                 // If the resource is a class just being loaded, decorate it
                 // with any attached transformers
 
@@ -2355,10 +2365,6 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
                     // Re-throw the original exception
                     throw e;
                 }
-                if (clazz == null) {
-                    // Not BZ 68721
-                    throw e;
-                }
             }
             entry.loadedClass = clazz;
         }
@@ -2470,7 +2476,32 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
         }
 
         char ch;
-        if (name.startsWith("javax")) {
+        if (name.startsWith("jakarta")) {
+            /* 7 == length("jakarta") */
+            if (name.length() == 7) {
+                return false;
+            }
+            ch = name.charAt(7);
+            if (isClassName && ch == '.') {
+                /* 8 == length("jakarta.") */
+                if (name.startsWith("servlet.jsp.jstl.", 8)) {
+                    return false;
+                }
+                if (name.startsWith("annotation.", 8) || name.startsWith("el.", 8) || name.startsWith("servlet.", 8) ||
+                        name.startsWith("websocket.", 8) || name.startsWith("security.auth.message.", 8)) {
+                    return true;
+                }
+            } else if (!isClassName && ch == '/') {
+                /* 8 == length("jakarta/") */
+                if (name.startsWith("servlet/jsp/jstl/", 8)) {
+                    return false;
+                }
+                if (name.startsWith("annotation/", 8) || name.startsWith("el/", 8) || name.startsWith("servlet/", 8) ||
+                        name.startsWith("websocket/", 8) || name.startsWith("security/auth/message/", 8)) {
+                    return true;
+                }
+            }
+        } else if (name.startsWith("javax")) {
             /* 5 == length("javax") */
             if (name.length() == 5) {
                 return false;
@@ -2478,20 +2509,12 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
             ch = name.charAt(5);
             if (isClassName && ch == '.') {
                 /* 6 == length("javax.") */
-                if (name.startsWith("servlet.jsp.jstl.", 6)) {
-                    return false;
-                }
-                if (name.startsWith("annotation.", 6) || name.startsWith("el.", 6) || name.startsWith("servlet.", 6) ||
-                        name.startsWith("websocket.", 6) || name.startsWith("security.auth.message.", 6)) {
+                if (name.startsWith("websocket.", 6)) {
                     return true;
                 }
             } else if (!isClassName && ch == '/') {
                 /* 6 == length("javax/") */
-                if (name.startsWith("servlet/jsp/jstl/", 6)) {
-                    return false;
-                }
-                if (name.startsWith("annotation/", 6) || name.startsWith("el/", 6) || name.startsWith("servlet/", 6) ||
-                        name.startsWith("websocket/", 6) || name.startsWith("security/auth/message/", 6)) {
+                if (name.startsWith("websocket/", 6)) {
                     return true;
                 }
             }
@@ -2539,8 +2562,6 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
     protected void addURL(URL url) {
         super.addURL(url);
         hasExternalRepositories = true;
-        // Clear the not found resources as they may now be available at the added URL.
-        notFoundClassResources.clear();
     }
 
 
