@@ -21,14 +21,17 @@ import java.nio.charset.StandardCharsets;
 import java.text.FieldPosition;
 import java.util.BitSet;
 import java.util.Date;
+import java.util.Map;
 
-import javax.servlet.http.HttpServletRequest;
+import jakarta.servlet.http.HttpServletRequest;
 
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.buf.MessageBytes;
+import org.apache.tomcat.util.descriptor.web.Constants;
 import org.apache.tomcat.util.http.parser.Cookie;
+import org.apache.tomcat.util.http.parser.HttpParser;
 import org.apache.tomcat.util.res.StringManager;
 
 public class Rfc6265CookieProcessor extends CookieProcessorBase {
@@ -77,8 +80,9 @@ public class Rfc6265CookieProcessor extends CookieProcessorBase {
             if (cookieValue != null && !cookieValue.isNull()) {
                 if (cookieValue.getType() != MessageBytes.T_BYTES) {
                     if (log.isDebugEnabled()) {
+                        Exception e = new Exception();
                         // TODO: Review this in light of HTTP/2
-                        log.debug(sm.getString("rfc6265CookieProcessor.expectedBytes"), new Exception());
+                        log.debug(sm.getString("rfc6265CookieProcessor.expectedBytes"), e);
                     }
                     cookieValue.toBytes();
                 }
@@ -87,8 +91,7 @@ public class Rfc6265CookieProcessor extends CookieProcessorBase {
                 }
                 ByteChunk bc = cookieValue.getByteChunk();
 
-                Cookie.parseCookie(bc.getBytes(), bc.getStart(), bc.getLength(), serverCookies,
-                        getCookiesWithoutEqualsInternal());
+                Cookie.parseCookie(bc.getBytes(), bc.getOffset(), bc.getLength(), serverCookies);
             }
 
             // search from the next position
@@ -98,13 +101,7 @@ public class Rfc6265CookieProcessor extends CookieProcessorBase {
 
 
     @Override
-    public String generateHeader(javax.servlet.http.Cookie cookie) {
-        return generateHeader(cookie, null);
-    }
-
-
-    @Override
-    public String generateHeader(javax.servlet.http.Cookie cookie, HttpServletRequest request) {
+    public String generateHeader(jakarta.servlet.http.Cookie cookie, HttpServletRequest request) {
 
         // Can't use StringBuilder due to DateFormat
         StringBuffer header = new StringBuffer();
@@ -117,7 +114,7 @@ public class Rfc6265CookieProcessor extends CookieProcessorBase {
         header.append(cookie.getName());
         header.append('=');
         String value = cookie.getValue();
-        if (value != null && !value.isEmpty()) {
+        if (value != null && value.length() > 0) {
             validateCookieValue(value);
             header.append(value);
         }
@@ -145,14 +142,14 @@ public class Rfc6265CookieProcessor extends CookieProcessorBase {
         }
 
         String domain = cookie.getDomain();
-        if (domain != null && !domain.isEmpty()) {
+        if (domain != null && domain.length() > 0) {
             validateDomain(domain);
             header.append("; Domain=");
             header.append(domain);
         }
 
         String path = cookie.getPath();
-        if (path != null && !path.isEmpty()) {
+        if (path != null && path.length() > 0) {
             validatePath(path);
             header.append("; Path=");
             header.append(path);
@@ -166,15 +163,53 @@ public class Rfc6265CookieProcessor extends CookieProcessorBase {
             header.append("; HttpOnly");
         }
 
-        SameSiteCookies sameSiteCookiesValue = getSameSiteCookies();
-
-        if (!sameSiteCookiesValue.equals(SameSiteCookies.UNSET)) {
+        String cookieSameSite = cookie.getAttribute(Constants.COOKIE_SAME_SITE_ATTR);
+        if (cookieSameSite == null) {
+            // Use processor config
+            SameSiteCookies sameSiteCookiesValue = getSameSiteCookies();
+            if (!sameSiteCookiesValue.equals(SameSiteCookies.UNSET)) {
+                header.append("; SameSite=");
+                header.append(sameSiteCookiesValue.getValue());
+            }
+        } else {
+            // Use explicit config
             header.append("; SameSite=");
-            header.append(sameSiteCookiesValue.getValue());
+            header.append(cookieSameSite);
         }
 
-        if (getPartitioned()) {
-            header.append("; Partitioned");
+        String cookiePartitioned = cookie.getAttribute(Constants.COOKIE_PARTITIONED_ATTR);
+        if (cookiePartitioned == null) {
+            if (getPartitioned()) {
+                header.append("; Partitioned");
+            }
+        } else {
+            if (Boolean.parseBoolean(cookiePartitioned)) {
+                header.append("; Partitioned");
+            }
+        }
+
+
+        // Add the remaining attributes
+        for (Map.Entry<String,String> entry : cookie.getAttributes().entrySet()) {
+            switch (entry.getKey()) {
+                case Constants.COOKIE_COMMENT_ATTR:
+                case Constants.COOKIE_DOMAIN_ATTR:
+                case Constants.COOKIE_MAX_AGE_ATTR:
+                case Constants.COOKIE_PATH_ATTR:
+                case Constants.COOKIE_SECURE_ATTR:
+                case Constants.COOKIE_HTTP_ONLY_ATTR:
+                case Constants.COOKIE_SAME_SITE_ATTR:
+                case Constants.COOKIE_PARTITIONED_ATTR:
+                    // Handled above so NO-OP
+                    break;
+                default: {
+                    validateAttribute(entry.getKey(), entry.getValue());
+                    header.append("; ");
+                    header.append(entry.getKey());
+                    header.append('=');
+                    header.append(entry.getValue());
+                }
+            }
         }
 
         return header.toString();
@@ -203,7 +238,7 @@ public class Rfc6265CookieProcessor extends CookieProcessorBase {
 
     private void validateDomain(String domain) {
         int i = 0;
-        int prev;
+        int prev = -1;
         int cur = -1;
         char[] chars = domain.toCharArray();
         while (i < chars.length) {
@@ -238,4 +273,19 @@ public class Rfc6265CookieProcessor extends CookieProcessorBase {
             }
         }
     }
+
+
+    private void validateAttribute(String name, String value) {
+        if (!HttpParser.isToken(name)) {
+            throw new IllegalArgumentException(sm.getString("rfc6265CookieProcessor.invalidAttributeName", name));
+        }
+
+        char[] chars = value.toCharArray();
+        for (char ch : chars) {
+            if (ch < 0x20 || ch > 0x7E || ch == ';') {
+                throw new IllegalArgumentException(
+                        sm.getString("rfc6265CookieProcessor.invalidAttributeValue", name, value));
+            }
+        }
+    }
 }
