@@ -184,8 +184,8 @@ public class WsWebSocketContainer implements WebSocketContainer, BackgroundProce
             ClientEndpointConfig clientEndpointConfiguration, URI path, Set<URI> redirectSet)
             throws DeploymentException {
 
-        if (log.isDebugEnabled()) {
-            log.debug(sm.getString("wsWebSocketContainer.connect.entry", clientEndpointHolder.getClassName(), path));
+        if (log.isTraceEnabled()) {
+            log.trace(sm.getString("wsWebSocketContainer.connect.entry", clientEndpointHolder.getClassName(), path));
         }
 
         boolean secure = false;
@@ -320,14 +320,14 @@ public class WsWebSocketContainer implements WebSocketContainer, BackgroundProce
             Future<Void> fHandshake = channel.handshake();
             fHandshake.get(timeout, TimeUnit.MILLISECONDS);
 
-            if (log.isDebugEnabled()) {
+            if (log.isTraceEnabled()) {
                 SocketAddress localAddress = null;
                 try {
                     localAddress = channel.getLocalAddress();
                 } catch (IOException ioe) {
                     // Ignore
                 }
-                log.debug(sm.getString("wsWebSocketContainer.connect.write", Integer.valueOf(request.position()),
+                log.trace(sm.getString("wsWebSocketContainer.connect.write", Integer.valueOf(request.position()),
                         Integer.valueOf(request.limit()), localAddress));
             }
             writeRequest(channel, request, timeout);
@@ -610,7 +610,12 @@ public class WsWebSocketContainer implements WebSocketContainer, BackgroundProce
         synchronized (endPointSessionMapLock) {
             Set<WsSession> sessions = endpointSessionMap.get(key);
             if (sessions != null) {
-                result.addAll(sessions);
+                // Some sessions may be in the process of closing
+                for (WsSession session : sessions) {
+                    if (session.isOpen()) {
+                        result.add(session);
+                    }
+                }
             }
         }
         return result;
@@ -1060,8 +1065,10 @@ public class WsWebSocketContainer implements WebSocketContainer, BackgroundProce
         if (backgroundProcessCount >= processPeriod) {
             backgroundProcessCount = 0;
 
+            // Check all registered sessions.
             for (WsSession wsSession : sessions.keySet()) {
                 wsSession.checkExpiration();
+                wsSession.checkCloseTimeout();
             }
         }
 
