@@ -438,7 +438,7 @@ public class TestBasicAuthParser {
      * for BASIC Authentication.
      * Note: only used internally, so no need to validate arguments.
      */
-    public static final class BasicAuthHeader {
+    private static final class BasicAuthHeader {
 
         private static final byte[] HEADER =
                 "authorization: ".getBytes(StandardCharsets.ISO_8859_1);
@@ -448,7 +448,7 @@ public class TestBasicAuthParser {
         /*
          * This method creates a valid base64 blob
          */
-        public BasicAuthHeader(String method, String username,
+        private BasicAuthHeader(String method, String username,
                 String password) {
             this(method, username, password, null);
         }
@@ -486,7 +486,7 @@ public class TestBasicAuthParser {
                         + ioe.getMessage());
             }
             // emulate tomcat server - offset points to method in header
-            authHeader.setStart(initialOffset);
+            authHeader.setOffset(initialOffset);
         }
 
         /*
@@ -506,7 +506,7 @@ public class TestBasicAuthParser {
                         + ioe.getMessage());
             }
             // emulate tomcat server - offset points to method in header
-            authHeader.setStart(initialOffset);
+            authHeader.setOffset(initialOffset);
         }
 
         /*
@@ -529,7 +529,7 @@ public class TestBasicAuthParser {
             }
         }
 
-        public ByteChunk getHeader() {
+        private ByteChunk getHeader() {
             return authHeader;
         }
     }
